# ================================================================

use strict;

# ================================================================

our($iLibraryVersion)='20211225';

# ================================================================

our($iSession_Directory)='/var/tmp/session/';

our($iSession_Lifetime)=7200;

# ================================================================

sub addRecordAndDeleteIfMatch
{
	my($aDBFile,$aIncrementalKey,$aKey,$aValue,$aAssociativeArray,$aIsEncodedValue)=@_;

	my(@tFileData)=&readFileAsText($aDBFile);
	my(%tFileQuery)=&getQueryEntries($tFileData[0],1);

	open(DATA_FILE,'>'.$aDBFile);
	flock(DATA_FILE,2);

	print DATA_FILE $aIncrementalKey.'='.($tFileQuery{$aIncrementalKey}+1);

	foreach(sort keys %$aAssociativeArray)
	{
		my($tKey)=$_;
		my($tValue)=$$aAssociativeArray{$tKey};

		if(&isDifferent($tKey,$aIncrementalKey))
		{
			if($aIsEncodedValue)
			{
				print DATA_FILE '&'.$tKey.'='.$tValue;
			}
			else
			{
				print DATA_FILE '&'.$tKey.'='.&encodeAsURL($tValue);
			}
		}
	}

	print DATA_FILE "\n";

	for(my($tCount1)=0;$tCount1<=($#tFileData);$tCount1++)
	{
		my(%tFileQuery)=&getQueryEntries($tFileData[$tCount1],1);

		if(&isDifferent($tFileQuery{$aKey},$aValue))
		{
			print DATA_FILE $tFileData[$tCount1];
		}
	}

	close(DATA_FILE);
}

# ----------------------------------------------------------------

sub convertUTF8ToEUCJP
{
	my($aString)=@_;

	return &convertXToY($aString,'utf8','euc-jp');
}

# ----------------------------------------------------------------

sub convertXToY
{
	my($aString,$aEncodingX,$aEncodingY)=@_;

	my($tResult)=$aString;

	use Encode;

	Encode::from_to($tResult,$aEncodingX,$aEncodingY);

	return $tResult;
}

# ----------------------------------------------------------------

sub copyFile
{
	my($aSource,$aDestination)=@_;

	my($tResult)=1;

	open(SOURCE_FILE,'<'.$aSource) or $tResult=0;
	open(DESTINATION_FILE,'>'.$aDestination) or $tResult=0;

	if($tResult)
	{
		binmode SOURCE_FILE;
		binmode DESTINATION_FILE;
		flock(SOURCE_FILE,1);
		flock(DESTINATION_FILE,2);

		my($tBuffer);

		while(read(SOURCE_FILE,$tBuffer,1048576)>0)
		{
			print DESTINATION_FILE $tBuffer;
		}
	}

	close(DESTINATION_FILE);
	close(SOURCE_FILE);

	return $tResult;
}

# ----------------------------------------------------------------

sub createSession
{
	my($tSessionID);

	while(1)
	{
		$tSessionID=&getRandomKey();

		if(!(-e $main::iSession_Directory.$tSessionID))
		{
			open(SESSION_FILE,'>'.$main::iSession_Directory.$tSessionID) or $tSessionID='0';
			close(SESSION_FILE);

			last;
		}
	}

	return $tSessionID;
}

# ----------------------------------------------------------------

sub decodeAsURL
{
	my($aTarget)=@_;

	my($tResult)=$aTarget;

	$tResult=~ s/%([0-9A-Fa-f][0-9A-Fa-f])/pack('C',hex($1))/eg;
	$tResult=~ s/\+/ /g;

	return $tResult;
}

# ----------------------------------------------------------------

sub deleteFile
{
	my($aTarget)=@_;

	my($tResult)=1;

	unlink($aTarget) or $tResult=0;

	return $tResult;
}

# ----------------------------------------------------------------

sub deleteOldSession
{
	my(@tFileList)=&getFileList($main::iSession_Directory,0);
	my($tCurrentTime)=time();

	for(my($tCount1)=0;$tCount1<=($#tFileList);$tCount1++)
	{
		my(@tFileStatus)=stat($main::iSession_Directory.$tFileList[$tCount1]);

		if($tFileStatus[9]<$tCurrentTime-$main::iSession_Lifetime)
		{
			&deleteFile($main::iSession_Directory.$tFileList[$tCount1]);
		}
	}
}

# ----------------------------------------------------------------

sub deleteRecordIfMatch
{
	my($aDBFile,$aKey,$aValue,$aIsEncodedValue)=@_;

	if(!($aIsEncodedValue))
	{
		$aValue=&encodeAsURL($aValue);
	}

	my(@tFileData)=&readFileAsText($aDBFile);

	open(DATA_FILE,'>'.$aDBFile);
	flock(DATA_FILE,2);

	for(my($tCount1)=0;$tCount1<=($#tFileData);$tCount1++)
	{
		my(%tFileQuery)=&getQueryEntries($tFileData[$tCount1],1);

		if(&isDifferent($tFileQuery{$aKey},$aValue))
		{
			print DATA_FILE $tFileData[$tCount1];
		}
	}

	close(DATA_FILE);
}

# ----------------------------------------------------------------

sub encodeAsURL
{
	my($aTarget)=@_;

	my($tResult)=$aTarget;

	$tResult=~ s/ /\+/g;
	$tResult=~ s/(\W)/sprintf("%%%02X",unpack('C',$1))/eg;

	return $tResult;
}

# ----------------------------------------------------------------

sub foldArrayWithScore
{
	my($aArray)=@_;

	my(%tResultArray)=();

	for(my($tCount1)=0;$tCount1<=($#$aArray);$tCount1++)
	{
		$tResultArray{$$aArray[$tCount1]}++;
	}

	return %tResultArray;
}

# ----------------------------------------------------------------

sub getAllRecord
{
	my($aDBFile,$aDoDecode)=@_;

	my(@tResult)=();

	my(@tFileData)=&readFileAsText($aDBFile);

	for(my($tCount1)=0;$tCount1<=($#tFileData);$tCount1++)
	{
		my(%tCurrentRecord)=&getQueryEntries($tFileData[$tCount1],$aDoDecode);

		push(@tResult,\%tCurrentRecord);
	}

	return @tResult;
}

# ----------------------------------------------------------------

sub getCookieEntries
{
	return &getStatementEntries($ENV{'HTTP_COOKIE'});
}

# ----------------------------------------------------------------

sub getCookieWithoutExpire
{
	my($aKey,$aValue)=@_;

	return 'Set-Cookie: '.$aKey.'='.$aValue.';'."\n";
}

# ----------------------------------------------------------------

sub getCurrentTimeString
{
	my($tSecond,$tMinute,$tHour,$tDay,$tMonth,$tYear,$tWeekDay)=localtime(time);

	$tYear+=1900;
	$tMonth++;

	my(@tWeek)=('Sun','Mon','Tue','Wed','Thu','Fri','Sat');

	return sprintf('%04d/%02d/%02d %s %02d:%02d:%02d',$tYear,$tMonth,$tDay,$tWeek[$tWeekDay],$tHour,$tMinute,$tSecond);
}

# ----------------------------------------------------------------

sub getDomainURL
{
	my($tResult)='';

	if(&isSame($ENV{'HTTPS'},'on'))
	{
		$tResult='https://';
	}
	else
	{
		$tResult='http://';
	}

	$tResult.=$ENV{'HTTP_HOST'};

	if(&isDifferent(($ENV{'SERVER_PORT'},'80'))&&(&isDifferent($ENV{'SERVER_PORT'},'443')))
	{
		$tResult.=':'.$ENV{'SERVER_PORT'};
	}

	return $tResult;
}

# ----------------------------------------------------------------

sub getDumpIfExists
{
	my($aTarget)=@_;

	my($tResult)='';

	if(-e $aTarget)
	{
		my(@tDumpData)=&readFileAsText($aTarget);

		for(my($tCount1)=0;$tCount1<=($#tDumpData);$tCount1++)
		{
			$tResult.=$tDumpData[$tCount1];
		}
	}

	return $tResult;
}

# ----------------------------------------------------------------

sub getFileList
{
	my($aDirectory,$aSortOrder)=@_;

	opendir DIRECTORY,$aDirectory;
	my(@tResult)=($aSortOrder==0)?(sort{$a cmp $b}(readdir DIRECTORY)):(sort{$b cmp $a}(readdir DIRECTORY));
	closedir DIRECTORY;

	for(my($tCount1)=0;$tCount1<=($#tResult);$tCount1++)
	{
		if(&isSame($tResult[$tCount1],'.'))
		{
			splice(@tResult,$tCount1,1);
		}

		if(&isSame($tResult[$tCount1],'..'))
		{
			splice(@tResult,$tCount1,1);
		}
	}

	return @tResult;
}

# ----------------------------------------------------------------

sub getHTMLAboutThisSoftware
{
	my($aApplicationName,$aApplicationVersion,$aLibraryVersion)=@_;

	my($tResult)='';

	$tResult.=&getHTMLCaption('ΥեȥˤĤ',1);
	$tResult.=&getHTMLParagraphOpen();
	$tResult.=$aApplicationName.'<br>'."\n";
	$tResult.='version '.$aApplicationVersion.' (+Library:'.$aLibraryVersion.')<br>'."\n";
	$tResult.=&getHTMLBreak();
	$tResult.='Programmed by Kei Kishiro'."\n";
	$tResult.=&getHTMLParagraphClose();

	return $tResult;
}

# ----------------------------------------------------------------

sub getHTMLBreak
{
	my($tResult)='';

	$tResult.='<br>'."\n";

	return $tResult;
}

# ----------------------------------------------------------------

sub getHTMLCaption
{
	my($aCaption,$aSize)=@_;

	my($tResult)='';

	$tResult.='<h'.$aSize.'>'.$aCaption.'</h'.$aSize.'>'."\n";

	return $tResult;
}

# ----------------------------------------------------------------

sub getHTMLDocumentClose
{
	my($tResult)='';

	$tResult.='</div>'."\n";
	$tResult.='</body>'."\n";
	$tResult.='</html>'."\n";

	return $tResult;
}

# ----------------------------------------------------------------

sub getHTMLDocumentCloseWithApplicationVersion
{
	my($aApplicationName,$aApplicationVersion,$aLibraryVersion)=@_;

	my($tResult)='';

	$tResult.=&getHTMLLine();
	$tResult.='<address>'.$aApplicationName.' version '.$aApplicationVersion.' (Library:'.$aLibraryVersion.') / Programmed by Kei Kishiro</address>'."\n";
	$tResult.='</div>'."\n";
	$tResult.='</body>'."\n";
	$tResult.='</html>'."\n";

	return $tResult;
}

# ----------------------------------------------------------------

sub getHTMLDocumentOpen
{
	my($aTitle,$aContentLanguage,$aCharset,$aCSSFiles,$aJavaScripts)=@_;

	my($tResult)='';

	$tResult.='<!doctype html public "-//W3C//DTD HTML 4.01 Transitional//EN">'."\n";
	$tResult.='<html lang="'.$aContentLanguage.'">'."\n";
	$tResult.='<head>'."\n";
	$tResult.='<meta name="viewport" content="width=device-width,initial-scale=1">'."\n";
	$tResult.='<meta name="ROBOTS" content="NOARCHIVE">'."\n";
	$tResult.='<meta http-equiv="Content-Type" content="text/html; charset='.$aCharset.'">'."\n";

	if(($#$aCSSFiles)>=0)
	{
		$tResult.='<style type="text/css">'."\n";

		for(my($tCount1)=0;$tCount1<=($#$aCSSFiles);$tCount1++)
		{
			$tResult.=&getDumpIfExists($$aCSSFiles[$tCount1]);
		}

		$tResult.='</style>'."\n";
	}

	if(($#$aJavaScripts)>=0)
	{
		$tResult.='<script type="text/javascript">'."\n";
		$tResult.='<!--'."\n";

		for(my($tCount1)=0;$tCount1<=($#$aJavaScripts);$tCount1++)
		{
			$tResult.=&getDumpIfExists($$aJavaScripts[$tCount1]);
		}

		$tResult.='//-->'."\n";
		$tResult.='</script>'."\n";
	}

	$tResult.='<title>'.$aTitle.'</title>'."\n";
	$tResult.='</head>'."\n";
	$tResult.='<body>'."\n";
	$tResult.='<div id="Outline">'."\n";

	return $tResult;
}

# ----------------------------------------------------------------

sub getHTMLFormClose
{
	my($tResult)='';

	$tResult.='</form>'."\n";

	return $tResult;
}

# ----------------------------------------------------------------

sub getHTMLFormOpen
{
	my($aAction,$aMethod,$aEncodeType)=@_;

	my($tResult)='';

	my($tEncodeType)='';

	if(&isSame($aEncodeType,''))
	{
		$tEncodeType='application/x-www-form-urlencoded';
	}
	else
	{
		$tEncodeType=$aEncodeType;
	}

	$tResult.='<form action="'.$aAction.'" method="'.$aMethod.'" enctype="'.$tEncodeType.'">'."\n";

	return $tResult;
}

# ----------------------------------------------------------------

sub getHTMLHiddenField
{
	my($aName,$aValue)=@_;

	my($tResult)='';

	$tResult.='<input type="hidden" name="'.$aName.'" value="'.$aValue.'">'."\n";

	return $tResult;
}

# ----------------------------------------------------------------

sub getHTMLLine
{
	my($tResult)='';

	$tResult.='<hr>'."\n";

	return $tResult;
}

# ----------------------------------------------------------------

sub getHTMLParagraphClose
{
	my($tResult)='';

	$tResult.='</p>'."\n";

	return $tResult;
}

# ----------------------------------------------------------------

sub getHTMLParagraphOpen
{
	my($tResult)='';

	$tResult.='<p>'."\n";

	return $tResult;
}

# ----------------------------------------------------------------

sub getHTMLPasswordField
{
	my($aName,$aValue,$aSize,$aMaxLength)=@_;

	my($tResult)='';

	$tResult.='<input type="password" name="'.$aName.'" value="'.$aValue.'" size="'.$aSize.'" maxlength="'.$aMaxLength.'"><br>'."\n";

	return $tResult;
}

# ----------------------------------------------------------------

sub getHTMLSubmitButtons
{
	my($aNameSubmit,$aNameReset)=@_;

	my($tResult)='';

	$tResult.='<input type="submit" value="'.$aNameSubmit.'">';

	if($aNameReset ne '')
	{
		$tResult.=' <input type="reset" value="'.$aNameReset.'">';
	}

	$tResult.='<br>'."\n";

	return $tResult;
}

# ----------------------------------------------------------------

sub getHTMLTextBox
{
	my($aName,$aValue,$aColSize,$aRowSize)=@_;

	my($tResult)='';

	$tResult.='<textarea ';

	if(&isDifferent($aName,''))
	{
			$tResult.='name="'.$aName.'"';
	}

	if(&isDifferent($aColSize,''))
	{
			$tResult.='cols="'.$aColSize.'"';
	}

	if(&isDifferent($aRowSize,''))
	{
			$tResult.='rows="'.$aRowSize.'"';
	}

	$tResult.='">';

	if(&isDifferent($aValue,''))
	{
			$tResult.=$aValue;
	}

	$tResult.='</textarea><br>'."\n";

	return $tResult;
}

# ----------------------------------------------------------------

sub getHTMLTextField
{
	my($aName,$aValue,$aSize,$aMaxLength)=@_;

	my($tResult)='';

	$tResult.='<input type="text" name="'.$aName.'" value="'.$aValue.'" size="'.$aSize.'" maxlength="'.$aMaxLength.'"><br>'."\n";

	return $tResult;
}

# ----------------------------------------------------------------

sub getHTMLUnknownAction
{
	my($tResult)='';

	$tResult.=&getHTMLCaption('顼ˤ',1);
	$tResult.=&getHTMLParagraphOpen();
	$tResult.='ꥯȤ̵뤵ޤ<br>'."\n";
	$tResult.='¸ߤʤ󤬻ꤵޤλְ꤬äƤޤ'."\n";
	$tResult.=&getHTMLParagraphClose();

	return $tResult;
}

# ----------------------------------------------------------------

sub getHTTPContentHeader
{
	my($aContentLanguage,$aCharset)=@_;

	my($tResult)='';

	$tResult.='Content-type: text/html; charset='.$aCharset."\n";
	$tResult.='Content-Language: '.$aContentLanguage."\n";
	$tResult.='Cache-Control: no-cache'."\n";

	return $tResult;
}

# ----------------------------------------------------------------

sub getHTTPResponse200Header
{
	my($tResult)='';

	$tResult.='HTTP/1.1 200 OK'."\n";
	$tResult.='Server: N/A'."\n";

	return $tResult;
}

# ----------------------------------------------------------------

sub getPostQueryEntries
{
	my($aDoDecode)=@_;

	my(%tReturn)=();

	if(&isSame($ENV{'REQUEST_METHOD'},'POST'))
	{
		read(STDIN,my($tPostString),$ENV{'CONTENT_LENGTH'});
		%tReturn=&getQueryEntries($tPostString,$aDoDecode);
	}

	return %tReturn;
}

# ----------------------------------------------------------------

sub getQuery
{
	my($aTarget)=@_;

	my($tBaseURL,$tQuery)=split(/\?/,$aTarget);

	return $tQuery;
}

# ----------------------------------------------------------------

sub getQueryEntries
{
	my($aString,$aDoDecode)=@_;

	my(@tQuery)=split(/\&/,$aString);
	my(%tReturn)=();

	for(my($tCount1)=0;$tCount1<=($#tQuery);$tCount1++)
	{
		my($tName,$tValue)=split(/=/,$tQuery[$tCount1]);
		chomp($tValue);

		if($aDoDecode)
		{
			$tReturn{$tName}=&decodeAsURL($tValue);
		}
		else
		{
			$tReturn{$tName}=$tValue;
		}
	}

	return %tReturn;
}

# ----------------------------------------------------------------

sub getRandomKey
{
	my($tKey)='';

	for(my($tCount1)=0;$tCount1<8;$tCount1++)
	{
		$tKey.=sprintf('%04x',int(rand(65536)));
	}

	return $tKey;
}

# ----------------------------------------------------------------

sub getRecordIfMatch
{
	my($aDBFile,$aKey,$aValue,$aDoDecode)=@_;

	my(@tResult)=();

	my(@tFileData)=&readFileAsText($aDBFile);

	for(my($tCount1)=0;$tCount1<=($#tFileData);$tCount1++)
	{
		my(%tCurrentRecord)=&getQueryEntries($tFileData[$tCount1],$aDoDecode);

		if(&isSame($tCurrentRecord{$aKey},$aValue))
		{
			push(@tResult,\%tCurrentRecord);
		}
	}

	return @tResult;
}

# ----------------------------------------------------------------

sub getRefererURLQueryEntries
{
	my($aDoDecode)=@_;

	return &getQueryEntries(&getQuery($ENV{'HTTP_REFERER'}),$aDoDecode);
}

# ----------------------------------------------------------------

sub getScriptURL
{
	return &getDomainURL().$ENV{'SCRIPT_NAME'};
}

# ----------------------------------------------------------------

sub getSessionVariable
{
	my($aSessionID,$aKey)=@_;

	my($tResult)='';

	my(@tFileData)=&readFileAsText($main::iSession_Directory.$aSessionID);

	for(my($tCount1)=0;$tCount1<=($#tFileData);$tCount1++)
	{
		my($tKey,$tValue)=split(/=/,$tFileData[$tCount1]);
		chomp($tValue);

		my($tDecodedKey)=&decodeAsURL($tKey);
		my($tDecodedtValue)=&decodeAsURL($tValue);

		if(&isSame($aKey,$tDecodedKey))
		{
			$tResult=$tDecodedtValue;

			last;
		}
	}

	return $tResult;
}

# ----------------------------------------------------------------

sub getStatementEntries
{
	my($aString)=@_;

	my(@tQuery)=split(/; */,$aString);
	my(%tReturn)=();

	for(my($tCount1)=0;$tCount1<=($#tQuery);$tCount1++)
	{
		my($tName,$tValue)=split(/=/,$tQuery[$tCount1]);
		chomp($tValue);
		$tReturn{$tName}=$tValue;
	}

	return %tReturn;
}

# ----------------------------------------------------------------

sub getURLQueryEntries
{
	my($aDoDecode)=@_;

	return &getQueryEntries($ENV{'QUERY_STRING'},$aDoDecode);
}

# ----------------------------------------------------------------

sub hasSame
{
	my($aTarget,$aTargetArray)=@_;

	for(my($tCount1)=0;$tCount1<=($#$aTargetArray);$tCount1++)
	{
		if(&isSame($$aTargetArray[$tCount1],$aTarget))
		{
			return 1;
		}
	}

	return 0;
}

# ----------------------------------------------------------------

sub isContained
{
	my($aString1,$aString2)=@_;

	if(index($aString1,$aString2)>=0)
	{
		return 1;
	}
	else
	{
		return 0;
	}
}

# ----------------------------------------------------------------

sub isDifferent
{
	my($aString1,$aString2)=@_;

	return ($aString1 ne $aString2);
}

# ----------------------------------------------------------------

sub isSame
{
	my($aString1,$aString2)=@_;

	return ($aString1 eq $aString2);
}

# ----------------------------------------------------------------

sub prepareHTTPResponse302
{
	my($aURL,$aExtraHTTPHeader)=@_;

	&prepareHTTPResponse30X('HTTP/1.1 302 Found',$aURL,$aExtraHTTPHeader);
}

# ----------------------------------------------------------------

sub prepareHTTPResponse30X
{
	my($aHTTPResponse,$aURL,$aExtraHTTPHeader)=@_;

	$main::gHeader='';
	$main::gBody='';

	$main::gHeader.=$aHTTPResponse."\n";
	$main::gHeader.='Server: N/A'."\n";

	chomp($aExtraHTTPHeader);

	if(&isDifferent($aExtraHTTPHeader,''))
	{
		$main::gHeader.=$aExtraHTTPHeader."\n";
	}

	$main::gHeader.='Location: '.$aURL."\n";
	$main::gHeader.='Content-type: text/html; charset=ASCII'."\n";
	$main::gHeader.='Cache-Control: no-cache'."\n";

	chomp($aExtraHTTPHeader);

	$main::gBody.='<html>'."\n";
	$main::gBody.='<body>'."\n";
	$main::gBody.='<a href="'.$aURL.'">'.$aURL.'</a>'."\n";
	$main::gBody.='</body>'."\n";
	$main::gBody.='</html>'."\n";
}

# ----------------------------------------------------------------

sub printResponseCache
{
	my($aHeader,$aBody)=@_;

	if((&isDifferent($aHeader,''))&&(&isDifferent($aBody,'')))
	{
		print $aHeader;
		print 'Content-Length: '.length($aBody)."\n\n";
		print $aBody;
	}
}

# ----------------------------------------------------------------

sub readFileAsText
{
	my($aTarget)=@_;

	open(DATA_FILE,'<'.$aTarget);
	flock(DATA_FILE,1);
	my(@tFileData)=<DATA_FILE>;
	close(DATA_FILE);

	return @tFileData;
}

# ----------------------------------------------------------------

sub replaceCharacterEUCJP
{
	my($aString,$aPattern,$aReplace)=@_;

	my($tResult)=$aString;

	my($tASCII)='[\x00-\x7F]';
	my($tTwoBytes)='[\x8E\xA1-\xFE][\xA1-\xFE]';
	my($tThreeBytes)='\x8F[\xA1-\xFE][\xA1-\xFE]';

	$tResult=~ s/\G((?:$tASCII|$tTwoBytes|$tThreeBytes)*?)(?:$aPattern)/$1$aReplace/g;

	return $tResult;
}

# ----------------------------------------------------------------

sub sanitize
{
	my($aTarget)=@_;

	my($tResult)=$aTarget;

	$tResult=~ s/\&/&amp;/g;
	$tResult=~ s/</&lt;/g;
	$tResult=~ s/>/&gt;/g;
	$tResult=~ s/"/&quot;/g;

	return $tResult;
}

# ----------------------------------------------------------------

sub separate
{
	my($aString)=@_;

	$aString=&replaceCharacterEUCJP($aString,"\r\n",' ');
	$aString=&replaceCharacterEUCJP($aString,'',' ');

	my(@tItemList)=split(/ /,$aString);

	my(@tResult)=();

	for(my($tCount1)=0;$tCount1<=($#tItemList);$tCount1++)
	{
		if(&isDifferent($tItemList[$tCount1],''))
		{
			push(@tResult,$tItemList[$tCount1]);
		}
	}

	return @tResult;
}

# ----------------------------------------------------------------

sub setSessionVariable
{
	my($aSessionID,$aKey,$aValue)=@_;

	my(@tFileData)=&readFileAsText($main::iSession_Directory.$aSessionID);

	open(DATA_FILE,'>'.$main::iSession_Directory.$aSessionID);
	flock(DATA_FILE,2);

	for(my($tCount1)=0;$tCount1<=($#tFileData);$tCount1++)
	{
		my($tKey,$tValue)=split(/=/,$tFileData[$tCount1]);
		chomp($tValue);

		my($tDecodedKey)=&decodeAsURL($tKey);

		if(&isDifferent($tDecodedKey,$aKey))
		{
			print DATA_FILE $tFileData[$tCount1];
		}
	}

	print DATA_FILE &encodeAsURL($aKey).'='.&encodeAsURL($aValue)."\n";

	close(DATA_FILE);
}

# ----------------------------------------------------------------

sub updateRecordIfMatch
{
	my($aDBFile,$aKey,$aValue,$aAssociativeArray,$aIsEncodedValue)=@_;

	if(!($aIsEncodedValue))
	{
		$aValue=&encodeAsURL($aValue);
	}

	my(@tFileData)=&readFileAsText($aDBFile);

	open(DATA_FILE,'>'.$aDBFile);
	flock(DATA_FILE,2);

	for(my($tCount1)=0;$tCount1<=($#tFileData);$tCount1++)
	{
		my(%tFileQuery)=&getQueryEntries($tFileData[$tCount1],0);

		foreach(sort keys %tFileQuery)
		{
			my($tKey)=$_;
			my($tValue)=$tFileQuery{$tKey};
		}

		if(&isSame($tFileQuery{$aKey},$aValue))
		{
			foreach(sort keys %$aAssociativeArray)
			{
				my($tKey)=$_;
				my($tValue)=$$aAssociativeArray{$tKey};

				if($aIsEncodedValue)
				{
					$tFileQuery{$tKey}=$tValue;
				}
				else
				{
					$tFileQuery{$tKey}=&encodeAsURL($tValue);
				}
			}

			my($tRecordLine)='';

			foreach(sort keys %tFileQuery)
			{
				my($tKey)=$_;
				my($tValue)=$tFileQuery{$tKey};

				if(&isDifferent($tValue,''))
				{
					if(&isDifferent($tRecordLine,''))
					{
						$tRecordLine.='&';
					}

					$tRecordLine.=$tKey.'='.$tValue;
				}
			}

			print DATA_FILE $tRecordLine."\n";
		}
		else
		{
			print DATA_FILE $tFileData[$tCount1];
		}
	}

	close(DATA_FILE);
}

# ----------------------------------------------------------------

sub updateSession
{
	my($aCurrentSessionID)=@_;

	my($tNewSessionID)='';

	if(-e $main::iSession_Directory.$aCurrentSessionID)
	{
		while(1)
		{
			$tNewSessionID=&getRandomKey();

			if(!(-e $main::iSession_Directory.$tNewSessionID))
			{
				if(!&copyFile($main::iSession_Directory.$aCurrentSessionID,$main::iSession_Directory.$tNewSessionID))
				{
					$tNewSessionID='';
				}
				
				last;
			}
		}
	}

	return $tNewSessionID;
}

# ================================================================

1;
